BLACKLIST = {
    # Builtins
    'builtins.open',
    'builtins.print',
    'builtins.input',
    # os module - File/Dir operations
    'os.remove',
    'os.unlink',
    'os.rmdir',
    'os.mkdir',
    'os.makedirs',
    'os.rename',
    'os.replace',
    'os.link',
    'os.symlink',
    'os.chmod',
    'os.chown',
    'os.utime',
    'os.truncate',
    'os.ftruncate',
    # os module - Process/System
    'os.system',
    'os.popen',
    'os.spawnl',
    'os.spawnle',
    'os.spawnlp',
    'os.spawnlpe',
    'os.spawnv',
    'os.spawnve',
    'os.spawnvp',
    'os.spawnvpe',
    'os.execl',
    'os.execle',
    'os.execlp',
    'os.execlpe',
    'os.execv',
    'os.execve',
    'os.execvp',
    'os.execvpe',
    'os.fork',
    'os.kill',
    'os.abort',
    'os.chdir',
    'os.fchdir',
    # os module - I/O
    'os.open',
    'os.read',
    'os.write',
    'os.close',
    'os.pipe',
    # sys module
    'sys.exit',
    'sys.stdout.write',
    'sys.stderr.write',
    # subprocess
    'subprocess.run',
    'subprocess.call',
    'subprocess.check_call',
    'subprocess.check_output',
    'subprocess.Popen',
    # shutil
    'shutil.rmtree',
    'shutil.copy',
    'shutil.copy2',
    'shutil.copyfile',
    'shutil.copytree',
    'shutil.move',
    'shutil.chown',
    # time
    'time.sleep',
    # random (state modification)
    'random.seed',
    # socket
    'socket.socket',
    'socket.create_connection',
    # pathlib (common write ops)
    'pathlib.Path.write_text',
    'pathlib.Path.write_bytes',
    'pathlib.Path.touch',
    'pathlib.Path.mkdir',
    'pathlib.Path.unlink',
    'pathlib.Path.rmdir',
    'pathlib.Path.rename',
    'pathlib.Path.replace',
}
